@@ -20,6 +20,7 @@ from AnyQt.QtCore import pyqtSignal as Signal
 import pyqtgraph as pg
 
 import Orange.data
+from Orange.data.util import get_unique_names
 import Orange.distance
 import Orange.misc
 from Orange.data import Table, Domain
@@ -34,6 +35,7 @@ from Orange.widgets.utils.graphicstextlist import TextListWidget
 from Orange.widgets.utils.graphicslayoutitem import SimpleLayoutItem
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg, Input, Output
 
 
@@ -132,6 +134,9 @@ class OWSilhouettePlot(widget.OWWidget):
         self._silhouette = None  # type: Optional[np.ndarray]
         self._silplot = None     # type: Optional[SilhouettePlot]
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         controllayout = self.controlArea.layout()
         assert isinstance(controllayout, QVBoxLayout)
         self._distances_gui_box = distbox = gui.widgetBox(
@@ -236,6 +241,9 @@ class OWSilhouettePlot(widget.OWWidget):
             self.data = data
 
     def handleNewSignals(self):
+        summary = len(self.data) if self.data else self.info.NoInput
+        details = format_summary_details(self.data) if self.data else ""
+        self.info.set_input_summary(summary, details)
         if not self._is_empty():
             self._update()
             self._replot()
@@ -494,12 +502,18 @@ class OWSilhouettePlot(widget.OWWidget):
                 scores = self._silhouette
 
             var = self.cluster_var_model[self.cluster_var_idx]
-            silhouette_var = Orange.data.ContinuousVariable(
-                "Silhouette ({})".format(escape(var.name)))
+            
+            domain = self.data.domain
+            proposed = "Silhouette ({})".format(escape(var.name))
+            names = [var.name for var in itertools.chain(domain.attributes,
+                                                         domain.class_vars,
+                                                         domain.metas)]
+            unique = get_unique_names(names, proposed)
+            silhouette_var = Orange.data.ContinuousVariable(unique)
             domain = Orange.data.Domain(
-                self.data.domain.attributes,
-                self.data.domain.class_vars,
-                self.data.domain.metas + (silhouette_var, ))
+                domain.attributes,
+                domain.class_vars,
+                domain.metas + (silhouette_var, ))
             data = self.data.transform(domain)
 
             if np.count_nonzero(selectedmask):
@@ -510,6 +524,9 @@ class OWSilhouettePlot(widget.OWWidget):
                 selected[:, silhouette_var] = np.c_[scores[selectedmask]]
             data[:, silhouette_var] = np.c_[scores]
 
+        summary = len(selected) if selected else self.info.NoOutput
+        details = format_summary_details(selected) if selected else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.selected_data.send(selected)
         self.Outputs.annotated_data.send(create_annotated_table(data, indices))
 
